/* Copyright 2016-2024 Michele Santullo
 * This file is part of "duckhandy".
 *
 * "duckhandy" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * "duckhandy" is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with "duckhandy".  If not, see <http://www.gnu.org/licenses/>.
 */

#include "catch2/catch_test_macros.hpp"
#include "duckhandy/implem/reversed_sized_array_bt.hpp"

TEST_CASE ("Check conversion functions in ReversedSizedArray", "[ReversedSizedArray][containers]") {
	using dhandy::ReversedSizedArray;

	{
		ReversedSizedArray<wchar_t, 8> ary;
		CHECK(ary.empty());
		CHECK(ary.size() == 0);
		ary.push_front('a');
		CHECK(not ary.empty());
		REQUIRE(ary.size() == 1);
		CHECK(ary.back() == 'a');
		CHECK(ary.to_string_view().size() == 1);
		CHECK(ary.to_string_view() == L"a");

		ary.push_front('n');
		ary.push_front('z');
		ary.push_front('*');
		REQUIRE(ary.size() == 4);
		CHECK(ary.to_string_view() == L"*zna");
	}

	{
		ReversedSizedArray<char, 4> ary;
		CHECK(ary.empty());
		CHECK(ary.size() == 0);
		ary.push_front('\0');
		CHECK(not ary.empty());
		REQUIRE(ary.size() == 1);
		CHECK(ary.back() == '\0');
		CHECK(ary.to_string_view().size() == 0);
		CHECK(ary.to_string_view() == "");

		ary.push_front('8');
		ary.push_front('+');
		ary.push_front('&');
		REQUIRE(ary.size() == 4);
		CHECK(ary.to_string_view() == "&+8");
	}
}
